﻿Type=Class
Version=5.9
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Custom View class
'#Event: ExampleEvent (Value As Int)
'#DesignerProperty: Key: BooleanExample, DisplayName: Boolean Example, FieldType: Boolean, DefaultValue: True, Description: Example of a boolean property.
'#DesignerProperty: Key: IntExample, DisplayName: Int Example, FieldType: Int, DefaultValue: 10, MinRange: 0, MaxRange: 100, Description: Note that MinRange and MaxRange are optional.
'#DesignerProperty: Key: StringWithListExample, DisplayName: String With List, FieldType: String, DefaultValue: Sunday, List: Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday
'#DesignerProperty: Key: StringExample, DisplayName: String Example, FieldType: String, DefaultValue: Text
'#DesignerProperty: Key: ColorExample, DisplayName: Color Example, FieldType: Color, DefaultValue: 0xFFCFDCDC, Description: You can use the built-in color picker to find the color values.
'#DesignerProperty: Key: DefaultColorExample, DisplayName: Default Color Example, FieldType: Color, DefaultValue: Null, Description: Setting the default value to Null means that a nullable field will be displayed.
Sub Class_Globals
	Private fx As JFX
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As Pane
	Private awesome As Font
	Private material As Font
	Private selectedFont As Font
	Private btnSearchIcon As Button
	Private optFontAwesome As RadioButton
	Private optMaterial As RadioButton
	Private txtSearchFor As TextField
	Private sSearch As String
	Private iconsMap As Map
	Private iconsMaterial As List
	Private iconsFontAwesome As List
	Private lstFonts As CustomListView
	Private btnIcon1 As Button
	Private btnIcon2 As Button
	Private btnIcon3 As Button
	Private btnIcon4 As Button
	Private btnIcon5 As Button
	Private btnIcon6 As Button
	Private btnIcon7 As Button
	Private lblFonts As Label
End Sub

Public Sub Initialize (Callback As Object, EventName As String)
	mEventName = EventName
	mCallBack = Callback
	awesome = fx.CreateFontAwesome(16)
	material = fx.CreateMaterialIcons(16)
	'read the fonts
	iconsMap = File.ReadMap(File.DirAssets,"icons.txt")
	'initialize the lists to load the fonts
	iconsMaterial.Initialize 
	iconsFontAwesome.Initialize
	'put the fonts in each list and sort them
	For Each strIcon As String In iconsMap.Keys 
		If strIcon.StartsWith("fa-") Then
			iconsFontAwesome.Add(strIcon)
		else if strIcon.StartsWith("mdi-") Then
			iconsMaterial.Add(strIcon)
		End If
	Next
	iconsFontAwesome.Sort(True)
	iconsMaterial.Sort(True)
End Sub

Public Sub DesignerCreateView (Base As Pane, Lbl As Label, Props As Map)
	mBase = Base
	mBase.LoadLayout("vFontGrid")
	optFontAwesome.Selected = True
	optMaterial.Selected = False
	lblFonts.Text=""
End Sub

Private Sub Base_Resize (Width As Double, Height As Double)
	
End Sub

Public Sub GetBase As Pane
	Return mBase
End Sub


Sub btnSearchIcon_Action
	'which font options are selected
	sSearch = txtSearchFor.Text.trim
	If optFontAwesome.Selected = True Then
		selectedFont = awesome
		LoadFonts(iconsFontAwesome,sSearch)
	else if optMaterial.Selected = True Then 
		selectedFont = material
		LoadFonts(iconsMaterial,sSearch)
	End If
End Sub

Sub LoadFonts(fromList As List, searchFor As String)
	'if we have a search term, lets filter the list
	Dim filterList As List
	filterList.Initialize 
	searchFor = searchFor.trim
	If searchFor.Length > 0 Then
		searchFor = searchFor.tolowercase
		For Each strItem As String In fromList
			If strItem.ToLowerCase.Contains(searchFor) Then
				filterList.Add(strItem)
			End If
		Next
	Else
		For Each strItem As String In fromList
			filterList.Add(strItem)
		Next
	End If
	'clear the fonts
	lstFonts.Clear
	If filterList.Size = 0 Then
		MsgBoxInform(Null,"Icon Search", "The icon you were searching for could not be found.")
		Return
	End If
	lblFonts.Text = "Fonts: " & filterList.size
	'we have 7 icons per row
	Dim totRows As Int = filterList.Size / 7
	'do we have left overs icons that will spill over
	Dim spillRow As Int = filterList.Size Mod 7
	'if we have a spill over, increment the rows
	If spillRow > 0 Then totRows = totRows + 1
	'let's build each row with the items found
	'i.e break a list into equal parts e.g 7 items
	Dim cntItems As Int = 0
	Dim strRow As StringBuilder
	Dim mRows As List
	mRows.Initialize 
	strRow.Initialize 
	For Each strItem As String In filterList
		cntItems = cntItems + 1
		If cntItems > 7 Then
			'start a new row
			strRow = RemDelim1(",",strRow)
			mRows.Add(strRow.ToString)
			strRow.Initialize
			cntItems = 0 
		Else
			strRow.Append(strItem).Append(",")
		End If		
	Next
	'there is a spill over, add the spill over items
	If spillRow > 0 Then 
		strRow = RemDelim1(",",strRow)
		mRows.Add(strRow.ToString)
	End If
	'add each row to the listview
	cntItems = 0
	For Each eachRow As String In mRows
		cntItems = cntItems + 1
		Dim pnlRow As B4XView = CreateListItem(eachRow,50,50)
		lstFonts.Add(pnlRow,cntItems)
	Next	
End Sub

Sub CreateListItem(Text As String, Width As Int, Height As Int) As B4XView
	'how many icons do we have in this row
	Dim spIcons() As String = StrParse(",",Text)
	Dim p As Pane
	p.Initialize("")
	p.LoadLayout("vFontButton")
	p.PrefHeight = Height
	'hide all the buttons
	btnIcon1.Visible = False
	btnIcon2.Visible = False
	btnIcon3.Visible = False
	btnIcon4.Visible = False
	btnIcon5.Visible = False
	btnIcon6.Visible = False
	btnIcon7.Visible = False
	'now lets display each icon
	Dim cntIcons As Int = 0
	For Each strIcon As String In spIcons
		cntIcons = cntIcons + 1
		Dim strIconValue As String = iconsMap.GetDefault(strIcon,"")
		strIconValue = strIconValue.touppercase
		Select Case cntIcons
			Case 1: SetButton(btnIcon1,strIcon,strIconValue)
			Case 2: SetButton(btnIcon2,strIcon,strIconValue)
			Case 3: SetButton(btnIcon3,strIcon,strIconValue)
			Case 4: SetButton(btnIcon4,strIcon,strIconValue)
			Case 5: SetButton(btnIcon5,strIcon,strIconValue)
			Case 6: SetButton(btnIcon6,strIcon,strIconValue)
			Case 7: SetButton(btnIcon7,strIcon,strIconValue)
		End Select
	Next
	Return p
End Sub

Sub SetButton(btnx As Button, iconname As String, iconvalue As String)
	btnx.Font = selectedFont
	btnx.Tag = CreateMap("iconname":iconname,"iconnumber":iconvalue)
	btnx.Visible = True
	btnx.Text = Chr(ParseInt(iconvalue))
	btnx.TooltipText = iconname
End Sub

Sub btnIcon_Action
	'which button has been clicked
	Dim btn As Button
	btn = Sender
	Dim fm As Map = btn.tag
	frmIcons.SelectedIconName = fm.GetDefault("iconname","")
	frmIcons.SelectedIconNumber = fm.GetDefault("iconnumber","")
	frmIcons.Close
End Sub

Sub MsgBoxInform(pform As Object, title As String, Msg As String)
	fx.Msgbox2(pform,Msg,title,"Ok","","",fx.MSGBOX_INFORMATION)
End Sub

public Sub RemDelim1(delimiter As String, value As StringBuilder) As StringBuilder
	If value.tostring.EndsWith(delimiter) = True Then
		Dim delimLen As Int = delimiter.length
		value.Remove(value.Length-delimLen,value.Length)
	End If
	Return value
End Sub


Sub ParseInt(strValue As String) As Int
	Try
		Return Bit.ParseInt(strValue, 16)
	Catch
		Return 0
	End Try
End Sub


Sub StrParse(Delimiter As String, MV As String) As String()
	Delimiter = FixDelimiter(Delimiter)
	Return Regex.Split(Delimiter, MV)
End Sub

Sub FixDelimiter(sValue As String) As String
	If sValue = "|" Then sValue = "\|"
	If sValue = "." Then sValue = "\."
	If sValue = "\" Then sValue = "\\"
	If sValue = "^" Then sValue = "\^"
	If sValue = "$" Then sValue = "\$"
	If sValue = "?" Then sValue = "\?"
	If sValue = "*" Then sValue = "\*"
	If sValue = "+" Then sValue = "\+"
	If sValue = "(" Then sValue = "\("
	If sValue = ")" Then sValue = "\)"
	If sValue = "[" Then sValue = "\["
	If sValue = "{" Then sValue = "\{"
	If sValue = ";" Then sValue = "\;"
	Return sValue
End Sub